/*
 * Copyright (C) 1999-2002 Wojciech Purczynski <cliph@isec.pl>
 *
 * THIS PROGRAM IS FREE SOFTWARE; YOU CAN REDISTRIBUTE IT AND/OR MODIFY
 * IT UNDER THE TERMS OF THE GNU GENERAL PUBLIC LICENSE AS PUBLISHED BY
 * THE FREE SOFTWARE  FOUNDATION;  EITHER  VERSION 2 OF THE LICENSE, OR
 * (AT YOUR OPTION) ANY LATER VERSION.
 *
 * THIS PROGRAM IS DISTRIBUTED IN THE HOPE THAT IT WILL BE USEFUL,
 * BUT WITHOUT ANY WARRANTY; WITHOUT EVEN THE IMPLIED  WARRANTY OF
 * MERCHANTABILITY OR FITNESS  FOR A PARTICULAR  PURPOSE.  SEE THE
 * GNU GENERAL PUBLIC LICENSE FOR MORE DETAILS.
 */

#include "module.h"

#include "config.h"
#include "lib.h"
#include "fs.h"
#include "rule.h"

#include <linux/spinlock.h>

static char buf[4096];
spinlock_t lock=SPIN_LOCK_UNLOCKED;

void log(const rule_t * r, int lvl, const char *fmt, ...)
{
	va_list args;
	int i;
	const char * exe;
	unsigned long flags;

	if (r->log_level == LOG_NOTHING) return;
	if (lvl < r->log_level) return;
	
	va_start(args, fmt);

	exe=get_exename();
	if (IS_ERR(exe)) exe=NULL;

	spin_lock_irqsave(&lock, flags);

	i=sprintf(buf, "%s%s%s: %.256s (%u:%u:%u) ",
	          lvl==LOG_SECURITY?KERN_WARNING:KERN_DEBUG,
		  NAME,
		  lvl==LOG_SECURITY?" SECURITY":"",
		  exe?exe:"[can't read executable's filename]",
		  PID, UID, EUID);
	
	if (exe) put_exename(exe);
	
	vsprintf(buf+i, fmt, args);
	printk("%.1022s\n", buf);
	
	spin_unlock_irqrestore(&lock, flags);
	
	return;
}
