/*
 * Copyright (C) 1999-2002 Wojciech Purczynski <cliph@isec.pl>
 *
 * THIS PROGRAM IS FREE SOFTWARE; YOU CAN REDISTRIBUTE IT AND/OR MODIFY
 * IT UNDER THE TERMS OF THE GNU GENERAL PUBLIC LICENSE AS PUBLISHED BY
 * THE FREE SOFTWARE  FOUNDATION;  EITHER  VERSION 2 OF THE LICENSE, OR
 * (AT YOUR OPTION) ANY LATER VERSION.
 *
 * THIS PROGRAM IS DISTRIBUTED IN THE HOPE THAT IT WILL BE USEFUL,
 * BUT WITHOUT ANY WARRANTY; WITHOUT EVEN THE IMPLIED  WARRANTY OF
 * MERCHANTABILITY OR FITNESS  FOR A PARTICULAR  PURPOSE.  SEE THE
 * GNU GENERAL PUBLIC LICENSE FOR MORE DETAILS.
 */

#ifndef _CAPSEL_LIB_H
#define _CAPSEL_LIB_H

#include <linux/config.h>
#include <linux/sched.h>
#include <linux/malloc.h>
#include <linux/smp_lock.h>

#include "config.h"
#include "rule.h"

#define UID 	(current->uid)
#define EUID 	(current->euid)
#define SUID	(current->suid)
#define GID 	(current->gid)
#define EGID 	(current->egid)
#define SGID	(current->sgid)
#define PID 	(current->pid)

/* 
 * It just checks the pointer and in the case of error it jumps to
 * label `label'. Error is stored in local `error' variable.
 */
#define __CHK_PTR(ptr, label)			\
	do {					\
		if (IS_ERR(ptr)) {		\
			rule_unlock();		\
			error=PTR_ERR(ptr);	\
			goto label;		\
		}				\
	} while (0)

/*
 * In the case of bad pointer we set `error' local variable and jump to
 * label `exit'.
 */
#define CHK_PTR(ptr) __CHK_PTR(ptr, exit)

#define mmalloc(size) kmalloc((size), GFP_KERNEL)
#define mfree(ptr)		\
	do {			\
		kfree(ptr);	\
		(ptr)=NULL;	\
	} while(0)

void log(const rule_t *, int, const char *, ...);

#ifdef KERNEL_2_4
typedef long rettype;
#define lock_kernel_2_2()
#define unlock_kernel_2_2()
#else
typedef int rettype;
/* lock_kernel & unlock_kernel used only in 2.2.x */
#define lock_kernel_2_2() lock_kernel()
#define unlock_kernel_2_2() unlock_kernel()
#endif /* KERNEL_2_4 */

#endif /* _CAPSEL_LIB_H */
